#ifndef _EESD_H_
#define _EESD_H_
#include<system.h>

/************************************************************
 * This is the header file that goes with the starting
 * library routines that  Drive LCD display
 *        
 */


/*  LCD Routines
 *
 *  Notes: Board is setup for 4 bit data on the upper
 *  bits of port D. Control port is on poert E.0 and
 *  E.1. Note that the R/W is pulled to 0, so that only
 *  writing is done to the display.
 *  NOTE: Ports must be set digital before using the LCD display
 *  */

// Ports naming

#define LCD_DATA           latd   // LCD data port
#define LCD_E_port         late   // enable control port
#define LCD_RS_port        late   // register select port

// Control signals

#define LCD_E_bit       1    // LCD enable register
#define LCD_RS_bit      0    // LCD register select

// Some usefull macros. These set or clear the control
// or register select bits.

#define LCD_E_U     set_bit(LCD_E_port, LCD_E_bit)
#define LCD_E_D     clear_bit(LCD_E_port, LCD_E_bit)
#define LCD_RS_U    set_bit(LCD_RS_port, LCD_RS_bit)
#define LCD_RS_D    clear_bit(LCD_RS_port, LCD_RS_bit)

// Use nested macros

#define write_data  LCD_RS_U; LCD_E_U; LCD_E_D;  // RS=1, toggle enable bit
#define write_cmd   LCD_RS_D; LCD_E_U; LCD_E_D;  // RS=0, toggle enable bit

// LCD Function prototypes 
void LCD_init(void);     // init LCD display
void LCD_ph1(char data);  // sends ascii of a single hex digit to display
void LCD_char(char data); // sends ascii character to display
void LCD_icmd(char data); //  sends 8 bit command (init till set to 4 bits)
void LCD_cmd(char data); // sends command to 4 bit interface nibble by nibble
void LCD_dec(char data); // displays byte as decimal number
void LCD_dec(short data);
void LCD_dec(unsigned short data);
void LCD_hex(char data);  // displays byte as hex number
void LCD_hex(short data);  // displays byte as hex number
void LCD_hex(unsigned short data);  // displays byte as hex number
void LCD_int(char data);
void LCD_int(short data);
void LCD_int(unsigned short data);
void LCD_nib(char data);
void LCD_bin(char data);  // displays byte as binary number
void LCD_bin(unsigned short data);  // displays byte as binary number
void LCD_init(void);     //initializes LDC display
void LCD_printf( const char* text ); //sends ascii string to display
void LCD_setpos(char line, char pos); //sets potition on display
void LCD_clear(void);    // clear LCD Screen


//Terminal Function prototypes
void term_init(void);  					//Initializes mc for serial i/o
void putc(char value); 					//send ascii character to terminal
char getc(void);	   					//get ascii character from terminal
void term_char(char data);				//prints a character to the terminal
void term_printf( const char* text );	//prints a character string to the terminal via the serial port
void term_dec(unsigned short dat);		//prints value of unsigned short variable to terminal
unsigned short getnum(void);			//prints value of unsigned short value
void term_menu(unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end);				//dumps memory contents into terminal
void dump_mem(unsigned short ad_high, unsigned short ad_low, unsigned short ad_high_end, unsigned short ad_low_end);

//Interrupts
void interrupt_init(void);				//automatically sets the proper interrupt flags
void interrupt_init_sleep(void);			

//Serial Protocols
void spi_init(void);																//establishes settings for SPI interface.
void i2c_init(void);																//establishes settings for I2C interface.

void i2c_start(void);																//These functions, internal to the 
void i2c_stop(void)	;																//main I2C functions, provide short						
void i2c_send_ack(void);															//hand for common actions.
void i2c_send_not_ack(void);
void i2c_receive_mode(void);
										
unsigned short i2c_time(bool write, unsigned short address_low, unsigned short address_high);		//displays time to LCD, with option of memory write															//reads time and prints to LCD (for the moment);
unsigned short mem_write_byte(unsigned short address_low, unsigned short address_high, short value); //writes voltage to memory chip and returns low memory
void time_init_clk( short minutes, short hours, short date, short month );			//initializes the clock to proper time
void time_init_set(void);															//Sets alarm and other setting registers on time chip
unsigned short address_update_low(unsigned short address_low);						//updates where the MC will write to memory next

//AD Conversion
void ad_init(void);							//sets register bits to allow AD conversion
unsigned short check_batt_voltage(void);			//gives battery voltage using AD pin reading
unsigned short check_batt_current(void);			//gives battery current using AD pin reading
unsigned short volt_scaler(unsigned short raw);		// Scales unsigned short raw data to range of 0-5
void print_volt(unsigned short dingus);				// Prints the scaled voltage to the LCD
unsigned short current_scaler(unsigned short raw);	// Scales unsigned short raw data from AD to range of 0-20 A
void print_amps(unsigned short dingus);				// Prints appropriately scaled voltage to LCD
void print_amps_term(unsigned short dingus);		// Prints a current to the HyperTerminal screen	
void print_volt_term(unsigned short dingus);		// Prints a voltage to the HyperTerminal screen
				
//Switching
void io_init(void);																												//sets pins to either digital inputs or outputs
unsigned short i2c_data(bool car_on, bool over, unsigned short fault, unsigned short address_low, unsigned short address_high);	//determines current electrical systems status
																																//pulls appropriate switches
//Killing Time
void sleep_mode(void);
short wait_mode(short mode);

#endif //_EESD_H_
