#include <system.h>
#include "50.h"

/*FINAL CONTROL ROUTINE, "PowerCop" Cruiser Power Regulation System
This software performs the following tasks:
	*Checks current being drawn off of auxilliary electrical devices at regular intervals using
	A/D conversion and the known I/V characteristics of our sensors.
	*Checks alternator current, battery current, and battery voltage to estimate whether the batttery is being
	charged or discharged, as well as providing a rough estimate of the state of charge.
	*Shuts off loads depending on the results of these measurements to prevent battery from being overdrawn
	*Writes the current being drawn from the loads, and any potential warnings about battery status and the
	time of each measurement, to an external EEPROM memory chip.
	*Provides a user interface to a series of buttons and an LCD screen, which allows users to set the system
	time, establish thresholds for the various device lines, and check the current being drawn at the moment by a
	given line.
	*Allows the user to connect the system to any PC with a serial port and HyperTerminal and upload the results
	stored in memory.  These results can be easily moved into an Excel spreadsheet.*/

#pragma DATA _CONFIG1H, _OSC_HS_1H //10 mhz
//#pragma DATA _CONFIG2H, _WDTPS_2048_2H 
//#pragma DATA _CONFIG2H, _WDTPS_16384_2H 
#pragma DATA _CONFIG4L, _LVP_OFF_4L 
#pragma DATA _CONFIG3H, _MCLRE_ON_3H

volatile bit user_int@INTCON3.0;
bool user_flag = false;

volatile bit time_out@INTCON.2;
bool lcd_menu_flag = false;

short timeoutflag = 0; 
short twentyfour = 0;
unsigned short over = 0;
unsigned short term_read = 0;

void interrupt(void)
{
	if (user_int)
	{
		user_int = 0;
		user_flag = true;
	}
	
    
    if (time_out)
    {
		time_out = 0;
		timeoutflag = 1;
    }         
	
}

void main(void)
{
	//ad_init();
	//io_init();
	adcon1 = 0x0f;
	ad_init();
	LCD_init();
	
	
	LCD_clear();
		unsigned short num = 22;
		/*If current on alternator is greater than some threshold (car on), set wait_time to
		wait_time_1.  If not (car is off), set it to wait_time_2.*/
		LCD_setpos(0,0);
		LCD_printf("Catch ");
		LCD_setpos(1,0);
		LCD_int(num);
		delay_s(1);
	while(1){}	
}



short check_timeoutflag(void){
	return timeoutflag;
	}
	
short write_timeoutflag(short tof){
	timeoutflag = tof;
	}

short check_twentyfour(void){
	return twentyfour;
	}

void write_twentyfour(short tf){
	twentyfour = tf;
	}
	
unsigned short check_override(void){
	return over;
	}

void write_override(unsigned short ovr){
	over = ovr;
}

unsigned short check_termread(){
	return term_read;
}

void write_term_read(unsigned short tr){
	term_read = tr;
}
