#include <system.h>
#include "EESD.h"

#pragma DATA _CONFIG1H, _OSC_HS_1H //10 mhz
#pragma DATA _CONFIG2H, _WDTPS_2048_2H
#pragma DATA _CONFIG2H, _WDTPS_16384_2H 
#pragma DATA _CONFIG4L, _LVP_OFF_4L 
#pragma DATA _CONFIG3H, _MCLRE_ON_3H

volatile bit alt@INTCON.1;	//ALTERNATOR INTERRUPT FLAG
volatile bit over@INTCON3.0;	//OVERRIDE INTERRUPT FLAG

bool altflag = false;		//Corresponding semaphores
bool overflag = false;

void interrupt(void)
{
	if (alt)
	{
		alt = 0;
		altflag = true;
	}
	if (over)
        {
		over = 0;
		overflag = true;
        }
}

void main(void)
{	
	int delay = 0;

	short minutes = 0x09;
	short hours = 0x13;
	short date = 0x29;
	short month = 0x03;

	interrupt_init();
	
	LCD_init();
	term_init();
	ad_init();
	i2c_init();
	io_init();
	time_init_clk(minutes, hours, date, month);
	time_init_set();
	
	short mode;
	short switch_status;
	short wait_time;
	short time_track = 0;
	short inc = 0;
	
	unsigned short volts;
	unsigned short volts_low;
	unsigned short volts_high;
	unsigned short amps_low;
	unsigned short amps_high;
	unsigned short amps;
	unsigned short raw_voltage;
	unsigned short raw_current;
	bool write = false;
	unsigned short write_length = 8;
	
	unsigned short address_low = 0xF0;
	unsigned short address_high = 0x00;
	unsigned short address_low_inc = 0xF0;
	unsigned short address_high_inc = 0x00;

	while(1){
		wdtcon.0 = 0;
		/*Check on memory position to avoid overwrite.*/
		if ( (address_low_inc + write_length) > 256){
			address_low_inc = 0;
			address_high_inc = address_high_inc + 1;
			}
			
        short bust = 0;
        time_track = 0;
		interrupt_init();
		
		/*Read the time from the time chip (I2C) and display on the LCD*/

		while (delay < 20){
			i2c_time(write, address_low_inc, address_high_inc);
			delay_ms(255);
			delay = delay + 1;
			LCD_clear();
		}
		delay = 0;
		write = true;
		address_low_inc = i2c_time(write, address_low_inc, address_high_inc);
		write = false;
		LCD_clear();

		/*Check and display voltage and current, as read off of the AD converter*/

		while (delay < 20){
			raw_voltage = check_batt_voltage();
			volts = volt_scaler(raw_voltage);
			print_volt(volts);
			raw_current = check_batt_current();
			LCD_setpos(1,0);
			amps = current_scaler(raw_current);
			print_amps(amps);
			delay_ms(255);
			delay = delay + 1;
			LCD_clear();
			}
		/*based on the measured voltage and current, pass the mode of the device and switch devices*/
		/*split up voltages and currents into 8-bit values*/
		
		volts_low = volts % 256;
		volts_high = volts / 256;
		amps_low = amps % 256;
		amps_high = amps / 256;
		
		/*write these bytes to memory*/
		
		address_low_inc = mem_write_byte(address_low_inc, address_high_inc, volts_high);
		address_low_inc = mem_write_byte(address_low_inc, address_high_inc, volts_low);
		address_low_inc = mem_write_byte(address_low_inc, address_high_inc, amps_high);
		address_low_inc = mem_write_byte(address_low_inc, address_high_inc, amps_low);
		
		mode = mode_select(volts, amps);
		switch_status = switch_devices(mode);
		delay = 0;
		
	//Enter a delay mode which spaces out measurements and writes.
		if(inc == 2){
			LCD_hex(address_low_inc);
			LCD_hex(address_high_inc);
			delay_ms(255);
			term_menu(address_high, address_low, address_high_inc, address_low_inc);
			address_low_inc = address_low;
			address_high_inc = address_high;
		}
		inc = inc + 1;
			
		wdtcon.0 = 1;	//enable watchdog timer
		alt = 0;		//ensures flag bits are cleared
		over = 0;		
		interrupt_init_sleep();
		delay_ms(255);
		if(portb.1 == 1){		//if car is on
			wait_time = 9;		//we end up waiting 4 + 1 = 5 minutes
			}
		if(portb.1 == 0){
			wait_time = 9;		//we end up waiting 9 + 1 = 10 minutes (cycles)
			}
		while( bust == 0){
			LCD_printf("time: ");
			LCD_int(time_track);
			sleep();
			LCD_clear();
			if(altflag == true){
				altflag = false;
				bust = 1;
				LCD_printf("car turned on or off!");
				}
			if(overflag == true){
				overflag = false;
				bust = 1;
				LCD_printf("override status changed!");
				}
			if(altflag == false && overflag == false){
				time_track = time_track + 1;
				LCD_printf(" timer reset");
				if(time_track >= wait_time){
					bust = 1;
					}
				}
			while (delay < 5){
			delay_ms(255);
			delay = delay + 1;
			}
			delay = 0;
			LCD_clear();
		}
	
	}
}	
	
